/**
 * This file is part of OpenMediaVault-autoshutdown.
 *
 * @license   http://www.gnu.org/licenses/gpl.html GPL Version 3
 * @author    Marcel Beck <marcel.beck@mbeck.org>
 * @copyright Copyright (c) 2012 Marcel Beck
 *
 * OpenMediaVault is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * OpenMediaVault is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenMediaVault. If not, see <http://www.gnu.org/licenses/>.
 */
	// require("js/omv/NavigationPanel.js")
	// require("js/omv/data/DataProxy.js")
	// require("js/omv/FormPanelExt.js")
	// require("js/omv/form/PasswordField.js")
	// require("js/omv/form/SharedFolderComboBox.js")
	// require("js/omv/form/plugins/FieldInfo.js")

Ext.ns("OMV.Module.Services");

// Register the menu.
OMV.NavigationPanelMgr.registerMenu("services", "autoshutdown", {
	text:"AutoShutdown",
	icon:"images/autoshutdown.png"
});

/**
 * @class OMV.Module.Services.AutoshutdownSettingsPanel
 * @derived OMV.FormPanelExt
 */
OMV.Module.Services.AutoshutdownSettingsPanel = function (config)
{
	var initialConfig = {
		rpcService  :"Autoshutdown",
		rpcGetMethod:"getSettings",
		rpcSetMethod:"setSettings"
	};
	Ext.apply(initialConfig, config);
	OMV.Module.Services.AutoshutdownSettingsPanel.superclass.constructor.call(
					this, initialConfig);
};
Ext.extend(OMV.Module.Services.AutoshutdownSettingsPanel, OMV.FormPanelExt, {
	getFormItems:function ()
	{
		return [
			{
				xtype   :"fieldset",
				title   :"General settings",
				defaults:{
					//				anchor: "100%",
					labelSeparator:""
				},
				items   :[
					{
						xtype     :"checkbox",
						name      :"enable",
						fieldLabel:"Enable",
						checked   :false,
						inputValue:1
					},
					{
						xtype     :"numberfield",
						name      :"flag",
						fieldLabel:"Flag",
						vtype     :"autoshutdown_flag",
						value     :6,
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Set the number of total failures before shutdown."
					},
					{
						xtype     :"numberfield",
						name      :"sleep",
						fieldLabel:"Sleep",
						vtype     :"autoshutdown_sleep",
						value     :180,
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Numbers of seconds between each check/loop."
					},
					{
						xtype     :"textfield",
						name      :"range",
						fieldLabel:"Range",
						vtype     :"autoshutdown_range",
						value     :"2..254",
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Define a range of IPs which should be scanned, "
										+ "via XXX.XXX.XXX.xxx last triple of IP address in a list. <br />"
										+ "The following scheme is mandatory v..v+m,w,x..x+n,y+o..y,z <br />"
										+ "  - define an ip range : <start>..<end> -> the two dots are mandatory <br />"
										+ "  - define a single ip : <ip> <br />"
										+ "  - all list entries are seperated by comma ',' <br />"
										+ "Please make sure to leave 1 and 255 out of the list !"
					},
					{
						xtype     :"checkbox",
						name      :"checkclockactive",
						fieldLabel:"Check Clock",
						checked   :false,
						inputValue:1,
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Check Clock to identify uptime."
					},
					{
						xtype   :"fieldset",
						title   :"Up Hours",
						plugins :[ OMV.form.plugins.FieldInfo ],
						infoText:"Define a range of hours, in which the system should stay up.",
						defaults:{
							labelSeparator:""
						},
						items   :[
							{
								xtype     :"compositefield",
								name      :"uphours",
								fieldLabel:"Hours",
								width     :200,
								items     :[
									{
										xtype        :"combo",
										name         :"uphours-begin",
										mode         :"local",
										store        :Array.range(0, 23),
										allowBlank   :false,
										editable     :false,
										triggerAction:"all",
										width        :50,
										value        :6,
										reset        :function ()
										{
										}
									},
									{
										xtype:"displayfield",
										value:".."
									},
									{
										xtype        :"combo",
										name         :"uphours-end",
										mode         :"local",
										store        :Array.range(0, 23),
										allowBlank   :false,
										editable     :false,
										triggerAction:"all",
										width        :50,
										value        :20,
										reset        :function ()
										{
										}
									}
								]
							}
						]
					},
					{
						xtype     :"textfield",
						name      :"nsocketnumbers",
						fieldLabel:"Network Socket Ports",
						vtype     :"autoshutdown_nsocket_numbers",
						value     :"22,139,445,10000,49152",
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Socket numbers to check for activity."
					},
					{
						xtype     :"textfield",
						name      :"loadprocnames",
						fieldLabel:"Proc Names",
						vtype     :"autoshutdown_proc_names",
						value     :"smbd,nfsd,transmission-daemon,mt-daapd",
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Command names of processes with load dependent children."
					},
					{
						xtype     :"textfield",
						name      :"tempprocnames",
						fieldLabel:"Temp Proc Names",
						vtype     :"autoshutdown_proc_names",
						value     :"in.tftpd",
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Command names of processes only started when active."
					},
					{
						xtype     :"checkbox",
						name      :"syslog",
						fieldLabel:"Log to Syslog",
						checked   :true,
						inputValue:1,
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Write log informations to system logs."
					},
					{
						xtype     :"checkbox",
						name      :"verbose",
						fieldLabel:"Verbose",
						checked   :true,
						inputValue:1,
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :"Verbose mode."
					},
					{
						xtype     :"textfield",
						name      :"extraoptions",
						fieldLabel:"Extra options",
						allowBlank:true,
						autoCreate:{
							tag         :"textarea",
							autocomplete:"off",
							rows        :"3",
							cols        :"65"
						},
						plugins   :[ OMV.form.plugins.FieldInfo ],
						infoText  :'<a href="https://github.com/OMV-Plugins/autoshutdown/blob/master/src/README">README</a>'
					}
				]
			}
		];
	}
});
OMV.NavigationPanelMgr.registerPanel("services", "autoshutdown", {
	cls     :OMV.Module.Services.AutoshutdownSettingsPanel,
	title   :"Settings",
	position:10
});

OMV.Module.Diagnostics.LogPlugin.Autoshutdown = function (config)
{
	var initialConfig = {
		title    :"AutoShutdown",
		stateId  :"92a5f193-e76a-481e-a1c0-12db308c97c0",
		columns  :[
			{
				header   :"Date & Time",
				sortable :true,
				dataIndex:"date",
				id       :"date",
				width    :40,
				renderer :OMV.util.Format.localeTimeRenderer()
			},
			{
				header   :"Type",
				sortable :true,
				dataIndex:"type",
				id       :"type",
				width    :20
			},
			{
				header   :"Log",
				sortable :true,
				dataIndex:"log",
				id       :"log"
			}
		],
		rpcArgs  :"autoshutdown",
		rpcFields:[
			{ name:"date" },
			{ name:"type" },
			{ name:"log" }
		]
	};
	Ext.apply(initialConfig, config);
	OMV.Module.Diagnostics.LogPlugin.Autoshutdown.superclass.constructor.call(
					this, initialConfig);
};

Ext.apply(Ext.form.VTypes, {

	autoshutdown_flag    :function (v)
	{
		return /^[0-9]{1,3}$/i.test(v);
	},
	autoshutdown_flagText:"Invalid Uri.",
	autoshutdown_flagMask:/[0-9]/i,

	autoshutdown_proc_names    :function (v)
	{
		return /^([A-Za-z0-9.-]{1,})+(,[A-Za-z0-9.-]{1,})*$/i.test(v);
	},
	autoshutdown_proc_namesText:"Invalid Uri.",
	autoshutdown_proc_namesMask:/[a-zA-Z0-9\-\.\,]/,

	autoshutdown_nsocket_numbers    :function (v)
	{
		return /^[0-9]{1,5}|[[0-9]{1,5}\,]$/i.test(v);
	},
	autoshutdown_nsocket_numbersText:"Invalid Uri.",
	autoshutdown_nsocket_numbersMask:/[0-9\,]/,

	autoshutdown_range     :function (v)
	{
		return /^([1-9]{1}[0-9]{0,2})?([1-9]{1}[0-9]{0,2}\.{2}[1-9]{1}[0-9]{0,2})?(,[1-9]{1}[0-9]{0,2})*((,[1-9]{1}[0-9]{0,2})\.{2}[1-9]{1}[0-9]{0,2})*$/i.test(v);
	},
	autoshutdown_rangeText :"Invalid Uri.",
	autoshutdown_rangesMask:/[0-9\.\,]/,

	autoshutdown_sleep    :function (v)
	{
		return /^[0-9]{1,3}$/i.test(v);
	},
	autoshutdown_sleepText:"Invalid Uri.",
	autoshutdown_sleepMask:/[0-9]/i

});

Ext.extend(OMV.Module.Diagnostics.LogPlugin.Autoshutdown,
					 OMV.Module.Diagnostics.LogPlugin, {
				});
OMV.preg("log", "autoshutdown", OMV.Module.Diagnostics.LogPlugin.Autoshutdown);